/*
 * JBoss, Home of Professional Open Source.
 *
 * Copyright 2023 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jboss.logging;

/**
 * An implementation of the {@linkplain LoggerProvider log provider} for the
 * {@linkplain java.util.logging.LogManager JDK Log Manager}.
 */
public final class JDKLoggerProvider extends AbstractMdcLoggerProvider implements LoggerProvider {

    @Override
    public Logger getLogger(final String name) {
        return new JDKLogger(name);
    }
}
